@@ -25,6 +25,8 @@
 #' * `vec_size(matrix)` == `vec_size(matrix[, i, drop = FALSE])`
 #' * `vec_size(vec_c(x, y))` == `vec_size(x)` + `vec_size(y)`
 #'
+#' @inheritParams rlang::args_error_context
+#'
 #' @param x,... Vector inputs or `NULL`.
 #' @param .size If `NULL`, the default, the output size is determined by
 #'   recycling the lengths of all elements of `...`. Alternatively, you can
@@ -64,6 +66,8 @@
 #' finding the common size because 1 is the identity of the recycling
 #' rules.
 #'
+#' @section Dependencies:
+#' - [vec_proxy()]
 #'
 #' @export
 #' @examples
@@ -77,19 +81,23 @@
 #'
 #' list_sizes(list("a", 1:5, letters))
 vec_size <- function(x) {
-  .Call(vctrs_size, x)
+  .Call(ffi_size, x, environment())
 }
 
 #' @export
 #' @rdname vec_size
-vec_size_common <- function(..., .size = NULL, .absent = 0L) {
-  .External2(vctrs_size_common, .size, .absent)
+vec_size_common <- function(...,
+                            .size = NULL,
+                            .absent = 0L,
+                            .arg = "",
+                            .call = caller_env()) {
+  .External2(ffi_size_common, .size, .absent)
 }
 
 #' @rdname vec_size
 #' @export
 list_sizes <- function(x) {
-  .Call(vctrs_list_sizes, x)
+  .Call(ffi_list_sizes, x, environment())
 }
 
 #' @rdname vec_size
@@ -142,3 +150,9 @@ vec_seq_along <- function(x) {
 vec_init_along <- function(x, y = x) {
   vec_slice(x, rep_len(NA_integer_, vec_size(y)))
 }
+
+vec_as_short_length <- function(n,
+                                arg = caller_arg(n),
+                                call = caller_env()) {
+  .Call(ffi_as_short_length, n, environment())
+}
