@@ -2,13 +2,14 @@
 #'
 #' @description
 #'
-#' \Sexpr[results=rd, stage=render]{vctrs:::lifecycle("experimental")}
+#' `r lifecycle::badge("experimental")`
 #'
 #' Convert `i` to the base type expected by [vec_as_location()] or
 #' [vec_as_location2()]. The values of the subscript type are
 #' not checked in any way (length, missingness, negative elements).
 #'
 #' @inheritParams vec_as_location
+#'
 #' @param logical,location,character How to handle logical, numeric,
 #'   and character subscripts.
 #'
@@ -29,26 +30,32 @@ vec_as_subscript <- function(i,
                              logical = c("cast", "error"),
                              numeric = c("cast", "error"),
                              character = c("cast", "error"),
-                             arg = NULL) {
-  if (!missing(...)) ellipsis::check_dots_empty()
+                             arg = NULL,
+                             call = caller_env()) {
+  check_dots_empty0(...)
 
   .Call(
-    vctrs_as_subscript,
+    ffi_as_subscript,
     i = i,
     logical = logical,
     numeric = numeric,
     character = character,
-    arg = arg
+    frame = environment()
   )
 }
-vec_as_subscript_result <- function(i, arg, logical, numeric, character) {
+vec_as_subscript_result <- function(i,
+                                    arg,
+                                    call,
+                                    logical,
+                                    numeric,
+                                    character) {
   .Call(
-    vctrs_as_subscript_result,
+    ffi_as_subscript_result,
     i = i,
     logical = logical,
     numeric = numeric,
     character = character,
-    arg = arg
+    frame = environment()
   )
 }
 
@@ -60,11 +67,13 @@ vec_as_subscript2 <- function(i,
                               logical = c("cast", "error"),
                               numeric = c("cast", "error"),
                               character = c("cast", "error"),
-                              arg = NULL) {
-  if (!missing(...)) ellipsis::check_dots_empty()
+                              arg = NULL,
+                              call = caller_env()) {
+  check_dots_empty0(...)
   result_get(vec_as_subscript2_result(
     i,
     arg,
+    call,
     logical = logical,
     numeric = numeric,
     character = character
@@ -72,16 +81,18 @@ vec_as_subscript2 <- function(i,
 }
 vec_as_subscript2_result <- function(i,
                                      arg,
+                                     call,
                                      logical = "cast",
                                      numeric = "cast",
                                      character = "cast") {
-  logical <- arg_match(logical, c("cast", "error"))
-  numeric <- arg_match(numeric, c("cast", "error"))
-  character <- arg_match(character, c("cast", "error"))
+  logical <- arg_match0(logical, c("cast", "error"))
+  numeric <- arg_match0(numeric, c("cast", "error"))
+  character <- arg_match0(character, c("cast", "error"))
 
   result <- vec_as_subscript_result(
     i,
     arg = arg,
+    call = call,
     logical = logical,
     numeric = numeric,
     character = character
@@ -103,8 +114,7 @@ vec_as_subscript2_result <- function(i,
       numeric = numeric,
       character = character,
       subscript_arg = arg,
-      body = bullets,
-      parent = result$err$parent
+      body = bullets
     )
 
     return(result)
@@ -150,11 +160,15 @@ as_opts_subscript2_type <- function(x, arg = NULL) {
 }
 
 
-stop_subscript <- function(i, ..., class = NULL) {
+stop_subscript <- function(i,
+                           ...,
+                           class = NULL,
+                           call = caller_env()) {
   abort(
     class = c(class, "vctrs_error_subscript"),
     i = i,
-    ...
+    ...,
+    call = vctrs_error_call(call)
   )
 }
 new_error_subscript <- function(class = NULL, i, ...) {
@@ -169,6 +183,7 @@ new_error_subscript_type <- function(i,
                                      numeric = "cast",
                                      character = "cast",
                                      ...,
+                                     call = NULL,
                                      class = NULL) {
   new_error_subscript(
     class = c(class, "vctrs_error_subscript_type"),
@@ -176,7 +191,8 @@ new_error_subscript_type <- function(i,
     logical = logical,
     numeric = numeric,
     character = character,
-    ...
+    ...,
+    call = vctrs_error_call(call)
   )
 }
 
@@ -271,28 +287,62 @@ cnd_body_subscript_dim <- function(cnd, ...) {
 cnd_subscript_element <- function(cnd, capital = FALSE) {
   elt <- cnd$subscript_elt %||% "element"
 
-  if (!is_string(elt, c("element", "row", "column"))) {
+  if (!is_string(elt, c("element", "row", "column", "table"))) {
     abort(paste0(
       "Internal error: `cnd$subscript_elt` must be one of ",
-      "`element`, `row`, or `column`."
+      "`element`, `row`, `column` or `table`."
     ))
   }
 
   if (capital) {
-    switch(elt,
+    switch(
+      elt,
       element = c("Element", "Elements"),
       row = c("Row", "Rows"),
-      column = c("Column", "Columns")
+      column = c("Column", "Columns"),
+      table = c("Table", "Tables")
     )
   } else {
     switch(elt,
       element = c("element", "elements"),
       row = c("row", "rows"),
-      column = c("column", "columns")
+      column = c("column", "columns"),
+      table = c("table", "tables")
     )
   }
 }
 
+cnd_subscript_element_cli <- function(n, cnd, capital = FALSE) {
+  elt <- cnd$subscript_elt %||% "element"
+
+  if (!is_string(elt, c("element", "row", "column", "table"))) {
+    abort(paste0(
+      "Internal error: `cnd$subscript_elt` must be one of ",
+      "`element`, `row`, `column` or `table`."
+    ))
+  }
+
+  if (capital) {
+    elt <- switch(
+      elt,
+      element = "Element{?s}",
+      row = "Row{?s}",
+      column = "Column{?s}",
+      table = "Table{?s}"
+    )
+  } else {
+    elt <- switch(
+      elt,
+      element = "element{?s}",
+      row = "row{?s}",
+      column = "column{?s}",
+      table = "table{?s}"
+    )
+  }
+
+  cli::pluralize("{n} ", elt)
+}
+
 subscript_actions <- c(
   "subset", "extract", "assign", "rename", "remove", "negate"
 )
