@@ -2,7 +2,7 @@
 #'
 #' @description
 #'
-#' \Sexpr[results=rd, stage=render]{vctrs:::lifecycle("defunct")}
+#' `r lifecycle::badge("defunct")`
 #'
 #' This function is defunct, please use [vec_is_empty()].
 #'
@@ -21,7 +21,7 @@ vec_empty <- function(x) {
 #'
 #' @description
 #'
-#' \Sexpr[results=rd, stage=render]{vctrs:::lifecycle("deprecated")}
+#' `r lifecycle::badge("deprecated")`
 #'
 #' These functions have been renamed:
 #'
@@ -54,10 +54,10 @@ vec_type2 <- function(x, y, ...) {
 #'
 #' @description
 #'
-#' \Sexpr[results=rd, stage=render]{vctrs:::lifecycle("soft-deprecated")}
+#' `r lifecycle::badge("deprecated")`
 #'
 #' `vec_as_index()` has been renamed to [vec_as_location()] and is
-#' soft-deprecated as of vctrs 0.2.2.
+#' deprecated as of vctrs 0.2.2.
 #'
 #' @inheritParams vec_as_location
 #'
@@ -72,8 +72,11 @@ vec_as_index <- function(i, n, names = NULL) {
   vec_assert(n, integer(), 1L)
   i <- vec_as_subscript(i)
 
+  # Picked up from the environment at the C level
+  arg <- NULL
+
   .Call(
-    vctrs_as_location,
+    ffi_as_location,
     i = i,
     n = n,
     names = names,
@@ -81,17 +84,17 @@ vec_as_index <- function(i, n, names = NULL) {
     loc_oob = "error",
     loc_zero = "remove",
     missing = "propagate",
-    arg = NULL
+    env = environment()
   )
 }
 
 #' Expand the length of a vector
 #'
 #' @description
-#' \Sexpr[results=rd, stage=render]{vctrs:::lifecycle("soft-deprecated")}
+#' `r lifecycle::badge("deprecated")`
 #'
 #' `vec_repeat()` has been replaced with [vec_rep()] and [vec_rep_each()] and is
-#' soft-deprecated as of vctrs 0.3.0.
+#' deprecated as of vctrs 0.3.0.
 #'
 #' @param x A vector.
 #' @param each Number of times to repeat each element of `x`.
