@@ -1,18 +1,11 @@
 # nocov start
 
-on_package_load <- function(pkg, expr) {
-  if (isNamespaceLoaded(pkg)) {
-    expr
-  } else {
-    thunk <- function(...) expr
-    setHook(packageEvent(pkg, "onLoad"), thunk)
-  }
-}
-
 .onLoad <- function(libname, pkgname) {
   check_linked_version(pkgname)
   ns <- ns_env("vctrs")
 
+  run_on_load()
+
   on_package_load("testthat", {
     s3_register("testthat::is_informative_error", "vctrs_error_cast_lossy")
     s3_register("testthat::is_informative_error", "vctrs_error_cast_lossy_dropped")
