@@ -1,17 +1,16 @@
 #include "vctrs.h"
-#include "cast.h"
+#include "type-data-frame.h"
 #include "type-factor.h"
-#include "utils.h"
+#include "type-tibble.h"
 
-// [[ include("cast.h") ]]
-SEXP vec_cast_dispatch(const struct cast_opts* opts,
-                       enum vctrs_type x_type,
-                       enum vctrs_type to_type,
-                       bool* lossy) {
-  SEXP x = opts->x;
-  SEXP to = opts->to;
-  struct vctrs_arg* x_arg = opts->x_arg;
-  struct vctrs_arg* to_arg = opts->to_arg;
+r_obj* vec_cast_dispatch_native(const struct cast_opts* opts,
+                                enum vctrs_type x_type,
+                                enum vctrs_type to_type,
+                                bool* lossy) {
+  r_obj* x = opts->x;
+  r_obj* to = opts->to;
+  struct vctrs_arg* x_arg = opts->p_x_arg;
+  struct vctrs_arg* to_arg = opts->p_to_arg;
 
   int dir = 0;
   enum vctrs_type2_s3 type2_s3 = vec_typeof2_s3_impl(x, to, x_type, to_type, &dir);
@@ -70,15 +69,48 @@ SEXP vec_cast_dispatch(const struct cast_opts* opts,
 
   case vctrs_type2_s3_dataframe_bare_tibble:
     if (dir == 0) {
-      return tib_cast(x, to, x_arg, to_arg);
+      return tib_cast(opts);
     } else {
       return df_cast_opts(opts);
     }
 
   case vctrs_type2_s3_bare_tibble_bare_tibble:
-    return tib_cast(x, to, x_arg, to_arg);
+    return tib_cast(opts);
 
   default:
-    return R_NilValue;
+    return r_null;
+  }
+}
+
+// [[ register() ]]
+r_obj* ffi_cast_dispatch_native(r_obj* x,
+                                r_obj* to,
+                                r_obj* fallback_opts,
+                                r_obj* x_arg,
+                                r_obj* to_arg,
+                                r_obj* frame) {
+  struct vctrs_arg c_x_arg = vec_as_arg(x_arg);
+  struct vctrs_arg c_to_arg = vec_as_arg(to_arg);
+  struct r_lazy call = { .x = syms_call, .env = frame };
+
+  struct cast_opts c_opts = new_cast_opts(x,
+                                          to,
+                                          &c_x_arg,
+                                          &c_to_arg,
+                                          call,
+                                          fallback_opts);
+
+  bool lossy = false;
+  r_obj* out = vec_cast_dispatch_native(&c_opts, vec_typeof(x), vec_typeof(to), &lossy);
+
+  if (lossy || out == r_null) {
+    return vec_cast_default(x,
+                            to,
+                            x_arg,
+                            to_arg,
+                            c_opts.call,
+                            &c_opts.fallback);
+  } else {
+    return out;
   }
 }
