@@ -1,3 +1,4 @@
+local_name_repair_quiet()
 
 # rows --------------------------------------------------------------------
 
@@ -6,12 +7,12 @@ test_that("empty inputs return an empty data frame", {
   expect_equal(vec_rbind(NULL, NULL), data_frame())
 })
 
-test_that("NULL is idempotent", {
+test_that("vec_rbind(): NULL is idempotent", {
   df <- data_frame(x = 1)
   expect_equal(vec_rbind(df, NULL), df)
 })
 
-test_that("output is tibble if any input is tibble", {
+test_that("vec_rbind() output is tibble if any input is tibble", {
   df <- data_frame(x = 1)
   dt <- tibble::tibble(x = 1)
 
@@ -28,7 +29,9 @@ test_that("type of column is common type of individual columns", {
   expect_equal(vec_rbind(x_int, x_int), data_frame(x = c(1L, 1L)))
   expect_equal(vec_rbind(x_int, x_dbl), data_frame(x = c(1, 2.5)))
 
-  expect_error(vec_rbind(x_int, x_chr), class = "vctrs_error_incompatible_type")
+  expect_snapshot({
+    (expect_error(vec_rbind(x_int, x_chr), class = "vctrs_error_incompatible_type"))
+  })
 })
 
 test_that("result contains union of columns", {
@@ -57,7 +60,8 @@ test_that("all inputs coerced to data frames", {
 })
 
 test_that("names are supplied if needed", {
-  expect_message(out <- vec_rbind(data_frame(...1 = 1), 1), "->")
+  local_name_repair_verbose()
+  expect_snapshot(out <- vec_rbind(data_frame(...1 = 1), 1))
   expect_equal(out, data_frame(...1 = c(1, 1)))
 })
 
@@ -77,6 +81,9 @@ test_that("can bind data.frame columns", {
 })
 
 test_that("can rbind unspecified vectors", {
+  expect_identical(vec_rbind(NA), data_frame(...1 = NA))
+  expect_identical(vec_rbind(NA, NA), data_frame(...1 = lgl(NA, NA)))
+
   df <- data.frame(x = 1)
   expect_identical(vec_rbind(NA, df), data.frame(x = c(NA, 1)))
   expect_identical(vec_rbind(df, NA), data.frame(x = c(1, NA)))
@@ -157,30 +164,27 @@ test_that("can rbind table objects (#913)", {
 })
 
 test_that("can rbind missing vectors", {
-  expect_identical(vec_rbind(na_int), data_frame(...1 = na_int))
-  expect_identical(vec_rbind(na_int, na_int), data_frame(...1 = int(na_int, na_int)))
-})
-
-test_that("can rbind unspecified vectors", {
-  expect_identical(vec_rbind(NA), data_frame(...1 = NA))
-  expect_identical(vec_rbind(NA, NA), data_frame(...1 = lgl(NA, NA)))
+  expect_identical(vec_rbind(c(x = na_int)), data_frame(x = na_int))
+  expect_identical(vec_rbind(c(x = na_int), c(x = na_int)), data_frame(x = int(na_int, na_int)))
 })
 
 test_that("vec_rbind() respects size invariants (#286)", {
   expect_identical(vec_rbind(), new_data_frame(n = 0L))
 
   expect_identical(vec_rbind(int(), int()), new_data_frame(n = 2L))
-  expect_identical(vec_rbind(int(), TRUE), new_data_frame(list(...1 = lgl(NA, TRUE))))
+  expect_identical(vec_rbind(c(x = int()), c(x = TRUE)), new_data_frame(list(x = lgl(NA, TRUE))))
 
   expect_identical(vec_rbind(int(), new_data_frame(n = 2L), int()), new_data_frame(n = 4L))
 })
 
 test_that("can repair names in `vec_rbind()` (#229)", {
-  expect_error(vec_rbind(.name_repair = "none"), "can't be `\"none\"`")
-  expect_error(vec_rbind(.name_repair = "minimal"), "can't be `\"minimal\"`")
+  expect_snapshot({
+    (expect_error(vec_rbind(.name_repair = "none"), "can't be `\"none\"`"))
+    (expect_error(vec_rbind(.name_repair = "minimal"), "can't be `\"minimal\"`"))
+    (expect_error(vec_rbind(list(a = 1, a = 2), .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique"))
+  })
 
   expect_named(vec_rbind(list(a = 1, a = 2), .name_repair = "unique"), c("a...1", "a...2"))
-  expect_error(vec_rbind(list(a = 1, a = 2), .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique")
 
   expect_named(vec_rbind(list(`_` = 1)), "_")
   expect_named(vec_rbind(list(`_` = 1), .name_repair = "universal"), c("._"))
@@ -200,7 +204,9 @@ test_that("can construct an id column", {
 })
 
 test_that("vec_rbind() fails with arrays of dimensionality > 3", {
-  expect_error(vec_rbind(array(NA, c(1, 1, 1))), "Can't bind arrays")
+  expect_snapshot({
+    (expect_error(vec_rbind(array(NA, c(1, 1, 1))), "Can't bind arrays"))
+  })
 })
 
 test_that("row names are preserved by vec_rbind()", {
@@ -218,14 +224,16 @@ test_that("can assign row names in vec_rbind()", {
   df1 <- mtcars[1:3, ]
   df2 <- mtcars[4:5, ]
 
-  expect_error(
-    vec_rbind(
-      foo = df1,
-      df2,
-      .names_to = NULL
-    ),
-    "specification"
-  )
+  expect_snapshot({
+    (expect_error(
+      vec_rbind(
+        foo = df1,
+        df2,
+        .names_to = NULL
+      ),
+      "specification"
+    ))
+  })
 
   # Combination
   out <- vec_rbind(
@@ -369,13 +377,25 @@ test_that("performance: Row binding with df-cols doesn't duplicate on every assi
 
 # cols --------------------------------------------------------------------
 
+test_that("vec_cbind() reports error context", {
+  expect_snapshot({
+    (expect_error(vec_cbind(foobar(list()))))
+    (expect_error(vec_cbind(a = 1:2, b = int())))
+  })
+})
+
 test_that("empty inputs give data frame", {
   expect_equal(vec_cbind(), data_frame())
   expect_equal(vec_cbind(NULL), data_frame())
   expect_equal(vec_cbind(data.frame(a = 1), NULL), data_frame(a = 1))
 })
 
-test_that("NULL is idempotent", {
+test_that("number of rows is preserved with zero column data frames (#1281)", {
+  df <- new_data_frame(n = 2L)
+  expect_size(vec_cbind(df, df), 2L)
+})
+
+test_that("vec_cbind(): NULL is idempotent", {
   df <- data_frame(x = 1)
   expect_equal(vec_cbind(df, NULL), df)
 })
@@ -402,12 +422,12 @@ test_that("matrix becomes data frame", {
 })
 
 test_that("duplicate names are de-deduplicated", {
-  expect_message(
-    expect_named(vec_cbind(x = 1, x = 1), c("x...1", "x...2")),
-    "x -> x...1",
-    fixed = TRUE
-  )
-  expect_named(vec_cbind(data.frame(x = 1), data.frame(x = 1)), c("x...1", "x...2"))
+  local_name_repair_verbose()
+
+  expect_snapshot({
+    (expect_named(vec_cbind(x = 1, x = 1), c("x...1", "x...2")))
+    (expect_named(vec_cbind(data.frame(x = 1), data.frame(x = 1)), c("x...1", "x...2")))
+  })
 })
 
 test_that("rows recycled to longest", {
@@ -427,7 +447,7 @@ test_that("rows recycled to longest", {
   )
 })
 
-test_that("output is tibble if any input is tibble", {
+test_that("vec_cbind() output is tibble if any input is tibble", {
   df <- data.frame(x = 1)
   dt <- tibble::tibble(y = 2)
 
@@ -437,14 +457,16 @@ test_that("output is tibble if any input is tibble", {
 })
 
 test_that("can override default .nrow", {
-  expect_dim(vec_cbind(1, .size = 3), c(3, 1))
+  expect_dim(vec_cbind(x = 1, .size = 3), c(3, 1))
 })
 
 test_that("can repair names in `vec_cbind()` (#227)", {
-  expect_error(vec_cbind(a = 1, a = 2, .name_repair = "none"), "can't be `\"none\"`")
+  expect_snapshot({
+    (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "none"), "can't be `\"none\"`"))
+    (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique"))
+  })
 
   expect_named(vec_cbind(a = 1, a = 2, .name_repair = "unique"), c("a...1", "a...2"))
-  expect_error(vec_cbind(a = 1, a = 2, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique")
 
   expect_named(vec_cbind(`_` = 1, .name_repair = "universal"), "._")
 
@@ -616,9 +638,10 @@ test_that("vec_cbind() consistently handles unnamed outputs", {
   )
 })
 
-test_that("rbind() and cbind() have informative outputs when repairing names", {
-  verify_output(test_path("output", "bind-name-repair.txt"), {
-    "# vec_rbind()"
+test_that("vec_rbind() name repair messages are useful", {
+  local_name_repair_verbose()
+
+  expect_snapshot({
     vec_rbind(1, 2)
     vec_rbind(1, 2, .names_to = NULL)
 
@@ -630,14 +653,20 @@ test_that("rbind() and cbind() have informative outputs when repairing names", {
 
     vec_rbind(c(a = 1), c(b = 2))
     vec_rbind(c(a = 1), c(b = 2), .names_to = NULL)
+  })
+})
 
-    "Silent when assigning duplicate row names of df-cols"
-    df <- new_data_frame(list(x = mtcars[1:3, 1, drop = FALSE]))
-    vec_rbind(df, df)
+test_that("vec_rbind() is silent when assigning duplicate row names of df-cols", {
+  df <- new_data_frame(list(x = mtcars[1:3, 1, drop = FALSE]))
 
-    vec_rbind(mtcars[1:4, ], mtcars[1:3, ])
+  expect_snapshot(vec_rbind(df, df))
+  expect_snapshot(vec_rbind(mtcars[1:4, ], mtcars[1:3, ]))
+})
 
-    "# vec_cbind()"
+test_that("vec_cbind() name repair messages are useful", {
+  local_name_repair_verbose()
+
+  expect_snapshot({
     vec_cbind(1, 2)
     vec_cbind(1, 2, ...10 = 3)
     vec_cbind(a = 1, b = 2)
@@ -717,21 +746,24 @@ test_that("vec_rbind() works with simple homogeneous foreign S4 classes", {
 })
 
 test_that("vec_rbind() fails with complex foreign S3 classes", {
-  verify_errors({
+  expect_snapshot({
     x <- structure(foobar(1), attr_foo = "foo")
     y <- structure(foobar(2), attr_bar = "bar")
-    expect_error(
+
+    (expect_error(
       vec_rbind(set_names(x, "x"), set_names(y, "x")),
       class = "vctrs_error_incompatible_type"
-    )
+    ))
   })
 })
 
 test_that("vec_rbind() fails with complex foreign S4 classes", {
-  verify_errors({
+  skip_if_cant_set_names_on_s4()
+
+  expect_snapshot({
     joe <- .Counts(1L, name = "Joe")
     jane <- .Counts(2L, name = "Jane")
-    expect_error(vec_rbind(joe, jane), class = "vctrs_error_incompatible_type")
+    (expect_error(vec_rbind(set_names(joe, "x"), set_names(jane, "y")), class = "vctrs_error_incompatible_type"))
   })
 })
 
@@ -847,22 +879,6 @@ test_that("vec_rbind() falls back to c() if S3 method is available for S4 class"
   expect_identical(out$x, .Counts(1:3, name = "dispatched"))
 })
 
-test_that("vec_cbind() and vec_rbind() have informative error messages", {
-  skip_if_cant_set_names_on_s4()
-
-  verify_output(test_path("error", "test-bind.txt"), {
-    "# vec_rbind() fails with complex foreign S3 classes"
-    x <- structure(foobar(1), attr_foo = "foo")
-    y <- structure(foobar(2), attr_bar = "bar")
-    vec_rbind(set_names(x, "x"), set_names(y, "x"))
-
-    "# vec_rbind() fails with complex foreign S4 classes"
-    joe <- .Counts(1L, name = "Joe")
-    jane <- .Counts(2L, name = "Jane")
-    vec_rbind(set_names(joe, "x"), set_names(jane, "x"))
-  })
-})
-
 test_that("rbind supports names and inner names (#689)", {
   skip_if(getRversion() >= "4.1.0", "work around r-devel bug")
   # Introduced in
@@ -904,7 +920,6 @@ test_that("vec_rbind() doesn't fall back to c() with proxied classes (#1119)", {
   foobar_rcrd <- function(x, y) new_rcrd(list(x = x, y = y), class = "vctrs_foobar")
 
   x <- foobar_rcrd(x = 1:2, y = 3:4)
-  y <- foobar_rcrd(x = 5L, y = 6L)
 
   out <- vec_rbind(x, x)
   exp <- data_frame(
@@ -946,11 +961,13 @@ test_that("can't zap names when `.names_to` is supplied", {
     vec_rbind(foo = c(x = 1), .names_to = zap(), .name_spec = zap()),
     data.frame(x = 1)
   )
-  expect_error(
-    vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap()),
-    "Can't zap outer names when `.names_to` is supplied.",
-    fixed = TRUE
-  )
+  expect_snapshot({
+    (expect_error(
+      vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap()),
+      "Can't zap outer names when `.names_to` is supplied.",
+      fixed = TRUE
+    ))
+  })
 })
 
 test_that("can zap outer names from a name-spec (#1215)", {
@@ -987,14 +1004,18 @@ test_that("column names are treated consistently in vec_rbind()", {
 
 # Golden tests -------------------------------------------------------
 
-test_that("rows-binding performs expected allocations", {
-  verify_output(test_path("performance", "test-bind.txt"), {
+test_that("row-binding performs expected allocations", {
+  vec_rbind_list <- function(x) {
+    vec_rbind(!!!x)
+  }
+
+  expect_snapshot({
     ints <- rep(list(1L), 1e2)
     named_ints <- rep(list(set_names(1:3, letters[1:3])), 1e2)
 
     "Integers as rows"
-    suppressMessages(with_memory_prof(vec_rbind(!!!ints)))
-    suppressMessages(with_memory_prof(vec_rbind(!!!named_ints)))
+    suppressMessages(with_memory_prof(vec_rbind_list(ints)))
+    suppressMessages(with_memory_prof(vec_rbind_list(named_ints)))
 
     "Data frame with named columns"
     df <- data_frame(
@@ -1003,17 +1024,17 @@ test_that("rows-binding performs expected allocations", {
       z = data_frame(Z = set_names(1:2, c("Za", "Zb")))
     )
     dfs <- rep(list(df), 1e2)
-    with_memory_prof(vec_rbind(!!!dfs))
+    with_memory_prof(vec_rbind_list(dfs))
 
     "Data frame with rownames (non-repaired, non-recursive case)"
     df <- data_frame(x = 1:2)
     dfs <- rep(list(df), 1e2)
     dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
-    with_memory_prof(vec_rbind(!!!dfs))
+    with_memory_prof(vec_rbind_list(dfs))
 
     "Data frame with rownames (repaired, non-recursive case)"
     dfs <- map(dfs, set_rownames_recursively)
-    with_memory_prof(vec_rbind(!!!dfs))
+    with_memory_prof(vec_rbind_list(dfs))
 
     # FIXME: The following recursive cases duplicate rownames
     # excessively because df-cols are restored at each chunk
@@ -1025,10 +1046,10 @@ test_that("rows-binding performs expected allocations", {
     )
     dfs <- rep(list(df), 1e2)
     dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
-    with_memory_prof(vec_rbind(!!!dfs))
+    with_memory_prof(vec_rbind_list(dfs))
 
     "FIXME (#1217): Data frame with rownames (repaired, recursive case)"
     dfs <- map(dfs, set_rownames_recursively)
-    with_memory_prof(vec_rbind(!!!dfs))
+    with_memory_prof(vec_rbind_list(dfs))
   })
 })
