@@ -3,11 +3,8 @@
 
 test_that("data frames print nicely", {
   expect_equal(vec_ptype_abbr(mtcars), "df[,11]")
-
-  verify_output(test_path("test-type-data-frame.txt"), {
-    vec_ptype_show(mtcars)
-    vec_ptype_show(iris)
-  })
+  expect_snapshot(vec_ptype_show(mtcars))
+  expect_snapshot(vec_ptype_show(iris))
 })
 
 test_that("embedded data frames print nicely", {
@@ -16,9 +13,7 @@ test_that("embedded data frames print nicely", {
   df$b <- list_of(1, 2, 3)
   df$c <- as_list_of(split(data.frame(x = 1:3, y = letters[1:3]), 1:3))
 
-  verify_output(test_path("test-type-data-frame-embedded.txt"), {
-    vec_ptype_show(df)
-  })
+  expect_snapshot(vec_ptype_show(df))
 })
 
 # coercing ----------------------------------------------------------------
@@ -62,22 +57,17 @@ test_that("combining data frames with foreign classes uses fallback", {
   expect_identical(vec_ptype_common(foo, foo, foo), foo)
   expect_incompatible_df(vec_ptype_common(foo, foo, df, foo), df)
 
-  expect_identical(
-    expect_df_fallback_warning(vec_ptype2_fallback(foo, df)),
-    new_fallback_df(df, c("vctrs_foobar", "data.frame"))
-  )
-  expect_identical(
-    expect_df_fallback_warning(vec_ptype2_fallback(df, foo)),
-    new_fallback_df(df, c("data.frame", "vctrs_foobar"))
-  )
-  expect_identical(
-    expect_df_fallback_warning(vec_ptype_common_df_fallback(foo, df)),
-    df
-  )
-  expect_identical(
-    expect_df_fallback_warning(vec_ptype_common_df_fallback(df, foo)),
-    df
-  )
+  expect_df_fallback_warning(res <- vec_ptype2_fallback(foo, df))
+  expect_identical(res, new_fallback_df(df, c("vctrs_foobar", "data.frame")))
+
+  expect_df_fallback_warning(res <- vec_ptype2_fallback(df, foo))
+  expect_identical(res, new_fallback_df(df, c("data.frame", "vctrs_foobar")))
+
+  expect_df_fallback_warning(res <- vec_ptype_common_df_fallback(foo, df))
+  expect_identical(res, df)
+
+  expect_df_fallback_warning(res <- vec_ptype_common_df_fallback(df, foo))
+  expect_identical(res, df)
 
   cnds <- list()
   withCallingHandlers(
@@ -105,21 +95,28 @@ test_that("combining data frames with foreign classes uses fallback", {
     df
   )
 
-  verify_errors({
-    foo <- structure(mtcars[1:3], class = c("foo", "data.frame"))
-    bar <- structure(mtcars[4:6], class = c("bar", "data.frame"))
-    baz <- structure(mtcars[7:9], class = c("baz", "data.frame"))
-    # Nested expect_warning() require testthat > 2.3.2
-    suppressWarnings(expect_warning(vec_ptype_common_df_fallback(foo, bar, baz)))
-    suppressWarnings(expect_warning(vec_ptype_common_df_fallback(foo, baz, bar, baz, foo, bar)))
+  foo <- structure(mtcars[1:3], class = c("foo", "data.frame"))
+  bar <- structure(mtcars[4:6], class = c("bar", "data.frame"))
+  baz <- structure(mtcars[7:9], class = c("baz", "data.frame"))
 
-    with_fallback_warning(expect_df_fallback_warning(invisible(vec_rbind(foo, data.frame(), foo))))
-    with_fallback_warning(expect_df_fallback_warning(invisible(vec_cbind(foo, data.frame(x = 1)))))
-    with_fallback_warning(expect_df_fallback_warning(invisible(vec_cbind(foo, data.frame(x = 1), bar))))
+  with_fallback_warning(expect_df_fallback_warning(invisible(vec_rbind(foo, data.frame(), foo))))
+  with_fallback_warning(expect_df_fallback_warning(invisible(vec_cbind(foo, data.frame(x = 1)))))
+  with_fallback_warning(expect_df_fallback_warning(invisible(vec_cbind(foo, data.frame(x = 1), bar))))
+  with_fallback_warning(expect_df_fallback_warning(invisible(vec_rbind(foo, baz, bar, baz, foo, bar))))
+
+  expect_snapshot({
+    vec_ptype_common_df_fallback(foo, bar, baz)
+    vec_ptype_common_df_fallback(foo, baz, bar, baz, foo, bar)
+
+    with_fallback_warning(invisible(vec_rbind(foo, data.frame(), foo)))
+    with_fallback_warning(invisible(vec_cbind(foo, data.frame(x = 1))))
+    with_fallback_warning(invisible(vec_cbind(foo, data.frame(x = 1), bar)))
+    with_fallback_warning(invisible(vec_rbind(foo, baz, bar, baz, foo, bar)))
 
     with_fallback_quiet(invisible(vec_rbind(foo, data.frame(), foo)))
     with_fallback_quiet(invisible(vec_cbind(foo, data.frame(x = 1))))
     with_fallback_quiet(invisible(vec_cbind(foo, data.frame(x = 1), bar)))
+    with_fallback_quiet(invisible(vec_rbind(foo, baz, bar, baz, foo, bar)))
   })
 })
 
@@ -233,6 +230,18 @@ test_that("casting to and from data frame preserves row names", {
   expect_identical(row.names(out), row.names(mtcars))
 })
 
+test_that("df_cast() evaluates arg lazily", {
+  expect_silent(df_cast(data_frame(), data_frame(), x_arg = print("oof")))
+  expect_silent(df_cast(data_frame(), data_frame(), to_arg = print("oof")))
+})
+
+# df_ptype2 ---------------------------------------------------------------
+
+test_that("df_ptype2() evaluates arg lazily", {
+  expect_silent(df_ptype2(data_frame(), data_frame(), x_arg = print("oof")))
+  expect_silent(df_ptype2(data_frame(), data_frame(), y_arg = print("oof")))
+})
+
 
 # new_data_frame ----------------------------------------------------------
 
@@ -365,21 +374,36 @@ test_that("n and row.names (#894)", {
 })
 
 test_that("`x` must be a list", {
-  expect_error(new_data_frame(1), "`x` must be a list")
+  expect_snapshot((expect_error(
+    new_data_frame(1),
+    "`x` must be a list"
+  )))
 })
 
 test_that("if supplied, `n` must be an integer of size 1", {
-  expect_error(new_data_frame(n = c(1L, 2L)), "must be an integer of size 1")
-  expect_error(new_data_frame(n = "x"), "must be an integer of size 1")
+  expect_snapshot({
+    (expect_error(new_data_frame(n = c(1L, 2L)), "must be an integer of size 1"))
+    (expect_error(new_data_frame(n = "x"), "must be an integer of size 1"))
+  })
+})
+
+test_that("if supplied, `n` can't be negative or missing (#1477)", {
+  expect_snapshot({
+    (expect_error(new_data_frame(n = -1L)))
+    (expect_error(new_data_frame(n = NA_integer_)))
+  })
 })
 
 test_that("`class` must be a character vector", {
-  expect_error(new_data_frame(class = 1), "must be NULL or a character vector")
+  expect_snapshot((expect_error(
+    new_data_frame(class = 1),
+    "must be NULL or a character vector"
+  )))
 })
 
 test_that("flatten info is computed", {
   df_flatten_info <- function(x) {
-    .Call(vctrs_df_flatten_info, x)
+    .Call(ffi_df_flatten_info, x)
   }
   expect_identical(df_flatten_info(mtcars), list(FALSE, ncol(mtcars)))
 
@@ -389,7 +413,7 @@ test_that("flatten info is computed", {
 
 test_that("can flatten data frames", {
   df_flatten <- function(x) {
-    .Call(vctrs_df_flatten, x)
+    .Call(ffi_df_flatten, x)
   }
   expect_identical(df_flatten(mtcars), mtcars)
 
@@ -397,10 +421,14 @@ test_that("can flatten data frames", {
   expect_identical(df_flatten(df), new_data_frame(list(x = 1, x = 2, x = 3, z = 4, z = 5)))
 })
 
-test_that("can flatten data frames with rcrd columns containing 1 field (#1318)", {
-  col <- new_rcrd(list(x = 1))
+test_that("can flatten data frames with rcrd columns (#1318)", {
+  col <- new_rcrd(list(a = 1))
   df <- data_frame(col = col, y = 1)
-  expect_identical(vec_proxy_equal(df), data_frame(x = 1, y = 1))
+  expect_identical(vec_proxy_equal(df), data_frame(col = 1, y = 1))
+
+  col <- new_rcrd(list(a = 1, b = 2))
+  df <- data_frame(col = col, y = 1)
+  expect_identical(vec_proxy_equal(df), data_frame(a = 1, b = 2, y = 1))
 })
 
 test_that("new_data_frame() zaps existing attributes", {
@@ -417,6 +445,16 @@ test_that("new_data_frame() zaps existing attributes", {
 
 # data_frame --------------------------------------------------------------
 
+test_that("data_frame() and df_list() report error context", {
+  expect_snapshot({
+    (expect_error(data_frame(a = 1, a = 1)))
+    (expect_error(data_frame(a = 1:2, b = int())))
+
+    (expect_error(df_list(a = 1, a = 1)))
+    (expect_error(df_list(a = 1:2, b = int())))
+  })
+})
+
 test_that("can construct data frames with empty input", {
   expect_identical(data_frame(), new_data_frame())
   expect_named(data_frame(), character())
@@ -433,7 +471,9 @@ test_that("input is tidy recycled", {
     data_frame(x = double(), y = integer())
   )
 
-  expect_error(data_frame(1:2, 1:3), class = "vctrs_error_incompatible_size")
+  expect_snapshot({
+    expect_error(data_frame(1:2, 1:3), class = "vctrs_error_incompatible_size")
+  })
 })
 
 test_that("dots are dynamic", {
@@ -487,24 +527,18 @@ test_that("`.size` can force a desired size", {
 })
 
 test_that("`.name_repair` repairs names", {
-  expect_named(
-    expect_message(data_frame(x = 1, x = 1, .name_repair = "unique")),
-    c("x...1", "x...2")
-  )
+  expect_message(res <- data_frame(x = 1, x = 1, .name_repair = "unique"))
+  expect_named(res, c("x...1", "x...2"))
 })
 
 test_that("`.name_repair` happens after auto-naming with empty strings", {
-  expect_named(
-    expect_message(data_frame(1, 2, .name_repair = "unique")),
-    c("...1", "...2")
-  )
+  expect_message(res <- data_frame(1, 2, .name_repair = "unique"))
+  expect_named(res, c("...1", "...2"))
 })
 
 test_that("`.name_repair` happens after splicing", {
-  expect_named(
-    expect_message(data_frame(x = 1, data_frame(x = 2), .name_repair = "unique")),
-    c("x...1", "x...2")
-  )
+  expect_message(res <- data_frame(x = 1, data_frame(x = 2), .name_repair = "unique"))
+  expect_named(res, c("x...1", "x...2"))
 })
 
 # fallback ----------------------------------------------------------------
@@ -518,14 +552,17 @@ test_that("data frame fallback handles column types (#999)", {
   expect_identical(vec_ptype2(df1, df2), common)
   expect_identical(vec_ptype2(df2, df1), common)
 
-  expect_error(
-    vec_ptype2(df1, df3),
-    class = "vctrs_error_incompatible_type"
-  )
-  expect_error(
-    vec_ptype2(df3, df1),
-    class = "vctrs_error_incompatible_type"
-  )
+  expect_snapshot({
+    local_error_call(call("my_function"))
+    (expect_error(
+      vec_ptype2(df1, df3),
+      class = "vctrs_error_incompatible_type"
+    ))
+    (expect_error(
+      vec_ptype2(df3, df1),
+      class = "vctrs_error_incompatible_type"
+    ))
+  })
 
   expect_identical(
     vec_cast(df1, df2),
@@ -570,47 +607,68 @@ test_that("falls back to tibble for tibble subclasses (#1025)", {
   expect_s3_class(expect_df_fallback_warning_maybe(vec_rbind(foo, mtcars, mtcars)), "tbl_df")
   expect_s3_class(expect_df_fallback_warning_maybe(vec_rbind(foo, mtcars, foobar(mtcars))), "tbl_df")
 
-  verify_errors({
-    with_fallback_warning(expect_df_fallback_warning(
-      vec_rbind(
+  with_fallback_warning(expect_df_fallback_warning(
+    vec_rbind(
+      foobar(tibble::as_tibble(mtcars)),
+      mtcars,
+      foobaz(mtcars)
+    )
+  ))
+  with_fallback_warning(expect_df_fallback_warning(
+    vec_rbind(
+      tibble::as_tibble(mtcars),
+      foobar(tibble::as_tibble(mtcars))
+    )
+  ))
+  with_fallback_warning(expect_df_fallback_warning(
+    vec_rbind(
+      foobar(tibble::as_tibble(mtcars)),
+      mtcars,
+      foobar(tibble::as_tibble(mtcars))
+    )
+  ))
+
+  expect_snapshot({
+    with_fallback_warning(
+      invisible(vec_rbind(
         foobar(tibble::as_tibble(mtcars)),
         mtcars,
         foobaz(mtcars)
-      )
-    ))
-    with_fallback_warning(expect_df_fallback_warning(
-      vec_rbind(
+      ))
+    )
+    with_fallback_warning(
+      invisible(vec_rbind(
         tibble::as_tibble(mtcars),
         foobar(tibble::as_tibble(mtcars))
-      )
-    ))
-    with_fallback_warning(expect_df_fallback_warning(
-      vec_rbind(
+      ))
+    )
+    with_fallback_warning(
+      invisible(vec_rbind(
         foobar(tibble::as_tibble(mtcars)),
         mtcars,
         foobar(tibble::as_tibble(mtcars))
-      )
-    ))
+      ))
+    )
 
     with_fallback_quiet(
-      vec_rbind(
+      invisible(vec_rbind(
         foobar(tibble::as_tibble(mtcars)),
         mtcars,
         foobaz(mtcars)
-      )
+      ))
     )
     with_fallback_quiet(
-      vec_rbind(
+      invisible(vec_rbind(
         tibble::as_tibble(mtcars),
         foobar(tibble::as_tibble(mtcars))
-      )
+      ))
     )
     with_fallback_quiet(
-      vec_rbind(
+      invisible(vec_rbind(
         foobar(tibble::as_tibble(mtcars)),
         mtcars,
         foobar(tibble::as_tibble(mtcars))
-      )
+      ))
     )
   })
 })
@@ -628,70 +686,3 @@ test_that("fallback is recursive", {
   exp <- new_data_frame(list(x = vec_rbind(df, df), y = c(NA, NA, NA, 1:3)))
   expect_incompatible_df(vec_rbind(foo, baz), exp)
 })
-
-test_that("data frame output is informative", {
-  verify_output(test_path("error", "test-type-data-frame.txt"), {
-    "# combining data frames with foreign classes uses fallback"
-    foo <- structure(mtcars[1:3], class = c("foo", "data.frame"))
-    bar <- structure(mtcars[4:6], class = c("bar", "data.frame"))
-    baz <- structure(mtcars[7:9], class = c("baz", "data.frame"))
-    vec_ptype_common_df_fallback(foo, bar, baz)
-    vec_ptype_common_df_fallback(foo, baz, bar, baz, foo, bar)
-
-    with_fallback_warning(invisible(vec_rbind(foo, data.frame(), foo)))
-    with_fallback_warning(invisible(vec_rbind(foo, baz, bar, baz, foo, bar)))
-
-    with_fallback_warning(invisible(vec_cbind(foo, data.frame(x = 1))))
-    with_fallback_warning(invisible(vec_cbind(foo, data.frame(x = 1), bar)))
-
-    with_fallback_quiet(invisible(vec_rbind(foo, data.frame(), foo)))
-    with_fallback_quiet(invisible(vec_rbind(foo, baz, bar, baz, foo, bar)))
-
-    with_fallback_quiet(invisible(vec_cbind(foo, data.frame(x = 1))))
-    with_fallback_quiet(invisible(vec_cbind(foo, data.frame(x = 1), bar)))
-
-    "# falls back to tibble for tibble subclasses (#1025)"
-    with_fallback_warning(
-      invisible(vec_rbind(
-        foobar(tibble::as_tibble(mtcars)),
-        mtcars,
-        foobaz(mtcars)
-      ))
-    )
-    with_fallback_warning(
-      invisible(vec_rbind(
-        tibble::as_tibble(mtcars),
-        foobar(tibble::as_tibble(mtcars))
-      ))
-    )
-    with_fallback_warning(
-      invisible(vec_rbind(
-        foobar(tibble::as_tibble(mtcars)),
-        mtcars,
-        foobar(tibble::as_tibble(mtcars))
-      ))
-    )
-
-    with_fallback_quiet(
-      invisible(vec_rbind(
-        foobar(tibble::as_tibble(mtcars)),
-        mtcars,
-        foobaz(mtcars)
-      ))
-    )
-
-    with_fallback_quiet(
-      invisible(vec_rbind(
-        tibble::as_tibble(mtcars),
-        foobar(tibble::as_tibble(mtcars))
-      ))
-    )
-    with_fallback_quiet(
-      invisible(vec_rbind(
-        foobar(tibble::as_tibble(mtcars)),
-        mtcars,
-        foobar(tibble::as_tibble(mtcars))
-      ))
-    )
-  })
-})
