@@ -1,5 +1,3 @@
-context("test-type-list-of")
-
 test_that("list_of inherits from list", {
   x1 <- list_of(1, 1)
   expect_s3_class(x1, "list")
@@ -42,36 +40,18 @@ test_that("is_list_of as expected", {
 
 test_that("print method gives human friendly output", {
   skip_on_cran() # Depends on tibble
-
-  x <- list_of(1, 2:3)
-
-  expect_known_output({
-      print(x)
-      cat("\n")
-      print(tibble::tibble(x))
-    },
-    file = test_path("test-list_of-print.txt")
-  )
+  expect_snapshot(list_of(1, 2:3))
+  expect_snapshot(tibble::tibble(x = list_of(1, 2:3)))
 })
 
 test_that("str method is reasonably correct", {
   x <- list_of(1, 2:3)
 
-  expect_known_output({
-      str(x)
-      cat("\n")
-      str(list(list(x, y = 2:1)))
-    },
-    file = test_path("test-list_of-str.txt")
-  )
+  expect_snapshot(str(x))
+  expect_snapshot(str(list(list(x, y = 2:1))))
 
-  expect_known_output({
-      str(x[0])
-      cat("\n")
-      str(list(list(x, y = 2:1)))
-    },
-    file = test_path("test-list_of-str-empty.txt")
-  )
+  expect_snapshot(str(x[0]))
+  expect_snapshot(str(list(list(x[0], y = 2:1))))
 })
 
 test_that("constructor requires list input", {
@@ -83,6 +63,15 @@ test_that("constructor requires size 0 ptype", {
   expect_error(new_list_of(ptype = 1), "must have size 0")
 })
 
+test_that("can combine a mix of named and unnamed list-ofs (#784)", {
+  a <- new_list_of(list(x = 1L), ptype = integer())
+  b <- new_list_of(list(2L), ptype = integer())
+
+  expect <- new_list_of(list(x = 1L, 2L), ptype = integer())
+
+  expect_identical(vec_c(a, b), expect)
+})
+
 # Subsetting --------------------------------------------------------------
 
 test_that("[ preserves type", {
@@ -144,12 +133,9 @@ test_that("list coercions are symmetric and unchanging", {
   mat <- maxtype_mat(types)
 
   expect_true(isSymmetric(mat))
-  expect_known_output(
-    mat,
-    test_path("test-list_of-type.txt"),
-    print = TRUE,
-    width = 200
-  )
+
+  local_options(width = 200)
+  expect_snapshot(print(mat))
 })
 
 test_that("max<list_of<a>, list_of<b>> is list_of<max<a, b>>", {
