@@ -1,5 +1,3 @@
-context("test-type")
-
 
 test_that("vec_ptype() is a no-op for NULL", {
   expect_null(vec_ptype(NULL))
@@ -49,10 +47,10 @@ test_that("non-missing logical get correct type", {
 })
 
 test_that("output tests", {
-  expect_known_output(vec_ptype_show(), "out/vec-ptype-0.txt")
-  expect_known_output(vec_ptype_show(integer()), "out/vec-ptype-1.txt")
-  expect_known_output(vec_ptype_show(integer(), double()), "out/vec-ptype-2.txt")
-  expect_known_output(vec_ptype_show(logical(), integer(), double()), "out/vec-ptype-3.txt")
+  expect_snapshot(vec_ptype_show())
+  expect_snapshot(vec_ptype_show(integer()))
+  expect_snapshot(vec_ptype_show(integer(), double()))
+  expect_snapshot(vec_ptype_show(logical(), integer(), double()))
 })
 
 test_that("vec_ptype_common() handles matrices", {
@@ -69,31 +67,29 @@ test_that("vec_ptype_common() includes index in argument tag", {
   large_df1 <- set_names(df1, nm)
   large_df2 <- set_names(df2, nm)
 
-  expect_known_output_nobang(file = test_path("test-type-vec-type-common-error.txt"), {
-    try2(vec_ptype_common(df1, df2))
-    try2(vec_ptype_common(df1, df1, df2))
-    try2(vec_ptype_common(large_df1, large_df2))
+  expect_snapshot(error = TRUE, vec_ptype_common(df1, df2))
+  expect_snapshot(error = TRUE, vec_ptype_common(df1, df1, df2))
+  expect_snapshot(error = TRUE, vec_ptype_common(large_df1, large_df2))
 
-    # Names
-    try2(vec_ptype_common(foo = TRUE, bar = "foo"))
-    try2(vec_ptype_common(foo = TRUE, baz = FALSE, bar = "foo"))
-    try2(vec_ptype_common(foo = df1, bar = df2))
-    try2(vec_ptype_common(df1, df1, bar = df2))
+  # Names
+  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, bar = "foo"))
+  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, baz = FALSE, bar = "foo"))
+  expect_snapshot(error = TRUE, vec_ptype_common(foo = df1, bar = df2))
+  expect_snapshot(error = TRUE, vec_ptype_common(df1, df1, bar = df2))
 
-    # One splice box
-    try2(vec_ptype_common(TRUE, !!!list(1, "foo")))
-    try2(vec_ptype_common(TRUE, !!!list(1, 2), "foo"))
-    try2(vec_ptype_common(1, !!!list(TRUE, FALSE), "foo"))
+  # One splice box
+  expect_snapshot(error = TRUE, vec_ptype_common(TRUE, !!!list(1, "foo")))
+  expect_snapshot(error = TRUE, vec_ptype_common(TRUE, !!!list(1, 2), "foo"))
+  expect_snapshot(error = TRUE, vec_ptype_common(1, !!!list(TRUE, FALSE), "foo"))
 
-    # One named splice box
-    try2(vec_ptype_common(foo = TRUE, !!!list(FALSE, FALSE), bar = "foo"))
-    try2(vec_ptype_common(foo = TRUE, !!!list(bar = 1, "foo")))
-    try2(vec_ptype_common(foo = TRUE, !!!list(bar = "foo")))
-    try2(vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), baz = "chr"))
+  # One named splice box
+  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(FALSE, FALSE), bar = "foo"))
+  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(bar = 1, "foo")))
+  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(bar = "foo")))
+  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), baz = "chr"))
 
-    # Two splice boxes in next and current
-    try2(vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), !!!list(baz = "chr")))
-  })
+  # Two splice boxes in next and current
+  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), !!!list(baz = "chr")))
 })
 
 test_that("proxied types are have s3 bare type", {
@@ -140,6 +136,10 @@ test_that("class_type() detects classes", {
   expect_identical(class_type(foobar(list())), "unknown")
   expect_identical(class_type(structure(list(), class = "list")), "list")
   expect_identical(class_type(subclass(structure(list(), class = "list"))), "list")
+  expect_identical(class_type(I(subclass(structure(list(), class = "list")))), "list")
+
+  expect_identical(class_type(I(list())), "bare_asis")
+  expect_identical(class_type(I(1)), "bare_asis")
 
   expect_identical(class_type(data.frame()), "bare_data_frame")
   expect_identical(class_type(tibble::tibble()), "bare_tibble")
@@ -192,19 +192,27 @@ test_that("the type of a classed data frame with an unspecified column retains u
   expect_identical(vec_ptype(df2), expect)
 })
 
+test_that("vec_ptype() methods can be written", {
+  local_methods(
+    vec_ptype.vctrs_foobar = function(x, ...) "dispatch"
+  )
+  expect_identical(vec_ptype(foobar()), "dispatch")
+})
+
 test_that("vec_ptype_finalise() works with NULL", {
   expect_identical(vec_ptype_finalise(NULL), NULL)
 })
 
 test_that("vec_ptype_finalise() works recursively over bare data frames", {
-  df <- data_frame(x = numeric(), y = unspecified(), z = partial_factor())
+  df <- new_data_frame(list(x = numeric(), y = unspecified(), z = partial_factor()))
   expect <- data_frame(x = numeric(), y = logical(), z = factor())
 
   expect_identical(vec_ptype_finalise(df), expect)
 })
 
 test_that("vec_ptype_finalise() works recursively over classed data frames", {
-  df <- subclass(data_frame(x = numeric(), y = unspecified(), z = partial_factor()))
+  df <- new_data_frame(list(x = numeric(), y = unspecified(), z = partial_factor()))
+  df <- subclass(df)
   expect <- subclass(data_frame(x = numeric(), y = logical(), z = factor()))
 
   expect_identical(vec_ptype_finalise(df), expect)
